# -*- coding: utf-8 -*-

#  PycaWM
#  button.py
#  Copyright (c) 2007 Vincent Rasneur, Anaël Verrier

#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; version 3 only.

#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.

#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import sys

if sys.version_info < (2, 5):
    raise ImportError('button module needs at least Python 2.5!')

from copy import copy

from Image import Image, open as image_open
from ImageFile import ImageFile

from types import MethodType
from Xlib import X

from basedirspec import load_first_data
from client import Client
from hookmanager import (MetaHookable, add_instance_post_hook,
                         remove_instance_post_hook)


class Button(object):
    __metaclass__ = MetaHookable
    stem_images = dict()

    def __init__(self, images, action_1=None, action_2=None, action_3=None,
                 action_4=None, action_5=None, client_p=None):
        if isinstance(images, dict):
            assert('default' in images)
        else:
            images = {'default': images}

        self.images = images
        self.image = None
        self.size = (0, 0)
        self.client = None
        self.load_images(images)
        for name, image in images.items():
            if isinstance(image, basestring):
                images[name] = self.stem_images[image]
        if action_1:
            self.action_1 = MethodType(action_1, self)
        if action_2:
            self.action_2 = MethodType(action_2, self)
        if action_3:
            self.action_3 = MethodType(action_3, self)
        if action_4:
            self.action_4 = MethodType(action_4, self)
        if action_5:
            self.action_5 = MethodType(action_5, self)
        if client_p:
            self.client_p = MethodType(client_p, self)

    @classmethod
    def load_images(cls, images):
        assert(isinstance(images, dict))

        for image in images.values():
            if not isinstance(image, basestring):
                continue
            if image not in cls.stem_images:
                cls.stem_images[image] = image_open(load_first_data(image))

    def press(self, client, mouse_button):
        self.__getattribute__('action_%d' % mouse_button)(client)

    def show(self, client, x, y, pixmap):
        pixmap.put_pil_image(client.gc, x, y, self.image)

    def action_1(self, client):
        pass

    def action_2(self, client):
        pass

    def action_3(self, client):
        pass

    def action_4(self, client):
        pass

    def action_5(self, client):
        pass

    def client_p(self, client):
        return True

    def set_image(self, name):
        self.image = self.images[name]
        self.size = self.image.size
        self.client.update_frame()

    def copy(self, client):
        button = copy(self)
        button.images = button.images.copy()
        for name, image in button.images.items():
            if not isinstance(image, (ImageFile, Image)):
                button.images[name] = image(client)
                if not isinstance(button.images[name], (ImageFile, Image)):
                    button.images[name] = button.images['default']
        button.image = button.images['default']
        button.size = button.image.size
        button.client = client
        return button

class PlopButton(Button):
    def action_2(self, client):
        print 'plop!'

    def action_4(self, client):
        print 'plop plop!'

class CloseButton(Button):
    def action_1(self, client):
        client.close()

class MiniIconButton(Button):
    def __init__(self, *args, **kwargs):
        Button.__init__(self, *args, **kwargs)
        self.images['miniicon'] = Client.get_mini_icon

    def copy(self, client):
        button = Button.copy(self, client)
        button.set_image('miniicon')
        return button

class IconifyButton(Button):
    def action_1(self, client):
        client.iconify()

class StatesButton(Button):
    def __init__(self, elements, *args, **kwargs):
        images = dict()
        for name, (image, _, _) in elements.items():
            images[name] = image
        self.hooks = dict()
        for _, hooked, hook in elements.values():
            self.hooks[hooked] = hook
        Button.__init__(self, images, *args, **kwargs)

    def copy(self, client):
        button = Button.copy(self, client)
        add_instance_post_hook(client.destroy, button.destroy)
        for hooked, hook in self.hooks.items():
            add_instance_post_hook(MethodType(hooked, client),
                                   MethodType(hook, button))
        return button

    def destroy(self, client):
        for hooked, hook in self.hooks.items():
            remove_instance_post_hook(MethodType(hooked, client),
                                      MethodType(hook, self))
        remove_instance_post_hook(client.destroy, self.destroy)

class ShadeButton(StatesButton):
    def __init__(self, shade_image, unshade_image):
        StatesButton.__init__(self, dict(
                default=(shade_image, Client.unshade, self.put_shade_image),
                unshade=(unshade_image, Client.shade, self.put_unshade_image)))

    @staticmethod
    def put_shade_image(button, *args, **kwargs):
        if not button.client.states.shaded:
            button.set_image('default')

    @staticmethod
    def put_unshade_image(button, *args, **kwargs):
        if button.client.states.shaded:
            button.set_image('unshade')

    def action_1(self, client):
        client.unshade() if client.states.shaded else client.shade()

    def copy(self, client):
        button = StatesButton.copy(self, client)
        if button.client.states.shaded:
            button.put_unshade_image()
        return button

class MaximizeButton(StatesButton):
    def __init__(self, maximize_image, unmaximize_image):
        StatesButton.__init__(self, dict(
                default=(maximize_image, Client.unmaximize,
                         self.put_maximize_image),
                unmaximize=(unmaximize_image, Client.maximize,
                            self.put_unmaximize_image)))

    @staticmethod
    def put_maximize_image(button, *args, **kwargs):
        if not button.client.states.maximized:
            button.set_image('default')

    @staticmethod
    def put_unmaximize_image(button, *args, **kwargs):
        if button.client.states.maximized:
            button.set_image('unmaximize')

    def action_1(self, client):
        client.unmaximize() if client.states.maximized else client.maximize()

    def action_2(self, client):
        client.unmaximize() if client.states.maximized \
            else client.maximize_vertically()

    def action_3(self, client):
        client.unmaximize() if client.states.maximized \
            else client.maximize_horizontally()

    def client_p(self, client):
        # no maximize button for transient clients
        return not client.states.transient

    def copy(self, client):
        button = StatesButton.copy(self, client)
        if button.client.states.maximized:
            button.put_unmaximize_image()
        return button
